﻿/* **************************************************************************************
*    Title: stabilizeClip.jsx
*    Author: David Torno
*    Date: 2019
*	
*	License
*	Released as open-source under the MIT license below.
*	The MIT License (MIT)
*	Copyright (c) 2019 David Torno
*	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
*	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
*	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.	
************************************************************************************** */
{
	function stabilizeClip(){
		///Comp data	--------------------------------------------------------------------------------------------------------------------------------
		var curComp, cName, cWidth, cHeight, cPAR, cDur, cFPS, cID;
		curComp = app.project.activeItem;
		cName = curComp.name;
		cWidth = curComp.width;
		cHeight = curComp.height;
		cPAR = curComp.pixelAspect;
		cDur = curComp.duration;
		cFPS = curComp.frameRate;
		cID = curComp.id;
		
		if(curComp instanceof CompItem){
			var selLayers, srcLayer;
			selLayers = curComp.selectedLayers;
			if(selLayers.length == 1){
				srcLayer = selLayers[0];
				///Add null to hold track keys	--------------------------------------------------------------------------------------------------------------------------------
				var mochaTrackLayer, mochaTrackLayerName, fxLayCtrl;
				mochaTrackLayer = curComp.layers.addNull(cDur);
				mochaTrackLayer.name = "Mocha Track Keys (PSR)";
				mochaTrackLayerName = mochaTrackLayer.name;
				mochaTrackLayer.enabled = false;
				fxLayCtrl = applyPluginEffect(mochaTrackLayer, "ADBE Layer Control");
				
				///Stabilizer layer data (srcLayer)	--------------------------------------------------------------------------------------------------------------------------------
				var stablizerLayer, stablizerLayerName;
				stablizerLayer = srcLayer;
				stablizerLayerName = stablizerLayer.name;
				
				///Add transform fx	--------------------------------------------------------------------------------------------------------------------------------
				var fxTransform;
				fxTransform = applyPluginEffect(stablizerLayer, "ADBE Geometry2");
				//Anchor Point
				fxTransform.property("ADBE Geometry2-0001").expression = "thisComp.layer(thisComp.layer(\""+mochaTrackLayerName+"\").effect(\"Layer Control\")(\"Layer\").index).toComp([0, 0, 0]);";
				//Position
				fxTransform.property("ADBE Geometry2-0002").setValue([curComp.width/2,curComp.height/2]);
				//Separate Scale
				fxTransform.property("ADBE Geometry2-0011").setValue(false);
				//Scale height
				fxTransform.property("ADBE Geometry2-0003").expression = "srcLayer = thisComp.layer(thisComp.layer(\""+mochaTrackLayerName+"\").effect(\"Layer Control\")(\"Layer\").index);\nconstantLayer = thisLayer;\ns = constantLayer.transform.scale.value;\nfor(var i=0; i<s.length; i++){\n\ts[i] *= 100/srcLayer.transform.scale.value[i];\n}\ns[1];";	//"srcLayer = thisComp.layer(effect(\"Layer Control\")(\"Layer\").index);\nconstantLayer = thisLayer;\ns = constantLayer.transform.scale.value;\nfor(var i=0; i<s.length; i++){\n\ts[i] *= 100/srcLayer.transform.scale.value[i];\n}\ns[1];";
				//Scale width
				fxTransform.property("ADBE Geometry2-0004").expression = "srcLayer = thisComp.layer(thisComp.layer(\""+mochaTrackLayerName+"\").effect(\"Layer Control\")(\"Layer\").index);\nconstantLayer = thisLayer;\ns = constantLayer.transform.scale.value;\nfor(var i=0; i<s.length; i++){\n\ts[i] *= 100/srcLayer.transform.scale.value[i];\n}\ns[0];";	//"srcLayer = thisComp.layer(effect(\"Layer Control\")(\"Layer\").index);\nconstantLayer = thisLayer;\ns = constantLayer.transform.scale.value;\nfor(var i=0; i<s.length; i++){\n\ts[i] *= 100/srcLayer.transform.scale.value[i];\n}\ns[0];";
				//Rotation
				fxTransform.property("ADBE Geometry2-0007").expression = "srcLayer = thisComp.layer(thisComp.layer(\""+mochaTrackLayerName+"\").effect(\"Layer Control\")(\"Layer\").index);\n-srcLayer.transform.rotation.value;"
				
				///Add offset fx	--------------------------------------------------------------------------------------------------------------------------------
				var posOffset, scaleOffset, rotOffset;
				
				posOffset = applyPluginEffect(stablizerLayer, "ADBE Point Control");	//Position offset
				posOffset.name = "Position Offset";
				posOffset.property("ADBE Point Control-0001").setValue([0.0, 0.0]);
				
//~ 				scaleOffset = applyPluginEffect(stablizerLayer, "ADBE Slider Control");	//Scale offset
//~ 				scaleOffset.name = "Scale Offset";
//~ 				scaleOffset.property("ADBE Slider Control-0001").setValue(0.0);
				
				rotOffset = applyPluginEffect(stablizerLayer, "ADBE Angle Control");	//Rotation offset
				rotOffset.name = "Rotation Offset";
				rotOffset.property("ADBE Angle Control-0001").setValue(0.0);

				mochaTrackLayer.property("ADBE Transform Group").property("ADBE Position").expression = "add(transform.position, thisComp.layer(\""+stablizerLayerName+"\").effect(\"Position Offset\")(\"Point\"));";
//~ 				mochaTrackLayer.property("ADBE Transform Group").property("ADBE Scale").expression = "var s = thisComp.layer(\""+stablizerLayerName+"\").effect(\"Scale Offset\")(\"Slider\");\nadd(transform.scale, [s, s]);";
				mochaTrackLayer.property("ADBE Transform Group").property("ADBE Rotate Z").expression = "(transform.rotation + thisComp.layer(\""+stablizerLayerName+"\").effect(\"Rotation Offset\")(\"Angle\"));";
				
				
				///Create working comp	--------------------------------------------------------------------------------------------------------------------------------
				var workCompName, workComp, curWCItm, projLen, wcName, wcWidth, wcHeight, wcPAR, wcDur, wcFPS, wcID, wcLayer;
				workCompName = cName+"_WorkHere";
				workComp = app.project.items.addComp(workCompName, cWidth, cHeight, cPAR, cDur, cFPS);
				workCompID = workComp.id;
				wcName = workCompName;
				wcWidth = workComp.width;
				wcHeight = workComp.height;
				wcPAR = workComp.pixelAspect;
				wcDur = workComp.duration;
				wcFPS = workComp.frameRate;
				wcID = workComp.id;
			
				projLen = app.project.numItems;
				for(var w=1; w<=projLen; w++){
					curWCItm = app.project.item(w);
					if(curWCItm.id == cID && curWCItm.name == cName){
						wcLayer = workComp.layers.add(curWCItm);
						wcLayer.collapseTransformation = true;
						break;
					}
				}
				
				workComp.openInViewer();
				
				///Create output comp	--------------------------------------------------------------------------------------------------------------------------------
				var outputCompName, outputComp, outputCompID, curOCItm, ocLayer;
				outputCompName = cName+"_Output";
				outputComp = app.project.items.addComp(outputCompName, cWidth, cHeight, cPAR, cDur, cFPS);
				outputCompID = outputComp.id;
				
				projLen = app.project.numItems;
				for(var o=1; o<=projLen; o++){
					curOCItm = app.project.item(o);
					if(curOCItm.id == wcID && curOCItm.name == wcName){
						ocLayer = outputComp.layers.add(curOCItm);
						ocLayer.collapseTransformation = true;
						break;
					}
				}
				
				if(outputComp instanceof CompItem){
					if(outputComp.layers.length > 0){
						var unstabilize = outputComp.layer(1);
						projLen = app.project.numItems;
						var curFComp = null;
						for(var f=1; f<=projLen; f++){
							curFItm = app.project.item(f);
							if(curFItm.id == cID && curFItm.name == cName){
								curFComp = curFItm.name;
								break;
							}
						}
						if(curFComp != null){
							unstabilize.property("ADBE Transform Group").property("ADBE Position").expression = "srcLayer = comp(\""+curFComp+"\").layer(comp(\""+curFComp+"\").layer(\""+mochaTrackLayerName+"\").effect(\"Layer Control\")(\"Layer\").index);\nsrcLayer.transform.position.value;";
							unstabilize.property("ADBE Transform Group").property("ADBE Scale").expression = "srcLayer = comp(\""+curFComp+"\").layer(comp(\""+curFComp+"\").layer(\""+mochaTrackLayerName+"\").effect(\"Layer Control\")(\"Layer\").index);\nsrcLayer.transform.scale.value;";
							unstabilize.property("ADBE Transform Group").property("ADBE Rotate Z").expression = "srcLayer = comp(\""+curFComp+"\").layer(comp(\""+curFComp+"\").layer(\""+mochaTrackLayerName+"\").effect(\"Layer Control\")(\"Layer\").index);\nsrcLayer.transform.rotation.value;";
						}
						outputComp.openInViewer();
					}
				}
			}else{
				alert("Only one source clip is allowed.");
			}
		}else{
			alert("Please select or open a composition first.");
		}
	}


	function applyPluginEffect(lay, fx){
		if(lay instanceof AVLayer || lay instanceof ShapeLayer || lay instanceof TextLayer){
			return lay.property("ADBE Effect Parade").addProperty(fx);
		}
	}

	app.beginUndoGroup("Stabilize");
		stabilizeClip();
	app.endUndoGroup();
}